# --------------------------------------------------------------------------
# Function and data for census data alone 
# --------------------------------------------------------------------------
censuslik <- function(pars,y){
  # This function returns the negative log-likelihood for the state-space 
  # immigration model
  phi1 <- expit(pars[1])
  phia <- expit(pars[2])
  rho <- exp(pars[3])
  im <- exp(pars[4])
  
  sigmasq <- 6
  TT <- matrix(c(0.5*rho*phi1,0.5*rho*phi1,phia+im,phia+im), nrow=2, ncol=2, byrow=TRUE) 
  Z <- matrix(c(1,1), nrow=1, ncol=2, byrow=TRUE) 
  V <- eigen(TT)
  vv <- t(t(V$vec[ ,1]))
  a1 <- matrix(c(5,10), nrow=2, ncol=1, byrow=TRUE)
  P1 <- matrix(c(0,0,0,0), nrow=2, ncol=2, byrow=TRUE)
  Qq <- matrix(c(0.5*rho*phi1,0.5*rho*phi1,(im+phia),(im+phia)), nrow=2, ncol=2, byrow=TRUE)
  H <- sigmasq
  
  l <-0
  for (i in 1:length(y)) { 
    Q <- diag((Qq %*% a1)[ ,1])
    a <- TT %*% a1
    P <- TT %*% P1 %*% t(TT) + Q
    v <- y[i]-Z %*% a
    FF <- Z %*% P %*% t(Z)+H
    M <- P %*% t(Z)
    a1 <- a + M %*% solve(FF)  %*% v
    P1 <- P - M %*% solve(FF) %*% t(M)
    
    l <- l - 1/2 * ( log(det(FF) ) + t(v) %*% solve(FF) %*% v)
  }
  
  return(-l)
}


# Census Data
y <-c(14,9,8,15,17,15,13,9,8,9,9,10,12,11,10,11,10,7,5,5,8,12,14,15,18,21)

# --------------------------------------------------------------------------
# Functions and data for integrated model combining census and CR data
# --------------------------------------------------------------------------
# Also need function census and census data given above
# The likelihood function needed for the integrated model is intlik 
# This function called CRlik and censuslik
# The two data sets are passed into intlik via a list
# replace y=y by datalist=list(y,m) to use optim
# and replace y=y by y=list(y,m) to use profileplot and hessianmethod

CRlik <- function(pars,m){
  # This function returns the negative log-likelihood of the CR model
  phi1 <- expit(pars[1])
  phia <- expit(pars[2])
  p <- expit(pars[3])
  
  N <- dim(m)[2]-1
  l <-0
  for (i in 1:N) { 
    Pjuvsum <-0
    Padsum <-0
    for (j in i:N) { 
      Pjuv=phi1*phia^(j-i)*(1-p)^(j-i)*p
      Pjuvsum=Pjuvsum+Pjuv
      l=l+m[i,j]*log(Pjuv)
      Pad=phia^(j-i+1)*(1-p)^(j-i)*p
      Padsum=Padsum+Pad
      l=l+m[i+N,j]*log(Pad)
      
    }
    l=l+m[i,N+1]*log(1-Pjuvsum)
    l=l+m[i+N,N+1]*log(1-Padsum)
  }
  
  return(-l)
}

intlik <- function(pars,datalist){
  # This function returns the negative log-likelihood of the integrated model
  # Combining census and CR data
  phi1logit <- pars[1]
  phialogit <- pars[2]
  plogit <- pars[3]
  rholog <-pars[4]
  imlog <- pars[5]
  y <- datalist[[1]]
  m <- datalist[[2]]
  pars1 <- c(phi1logit,phialogit,rholog,imlog)
  pars2 <- c(phi1logit,phialogit,plogit)
  l <- do.call("censuslik",list(pars1,y))+do.call("CRlik",list(pars2,m))
  
  return(l)
}

# Capture-recapture data
m <- matrix(c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,13,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,
              0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,
              0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,
              0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,22,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,
              0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,9,
              0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,15,
              0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,14,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,3,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,4,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,11,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,13,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,15,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,15,
              1,2,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,6,
              0,3,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,
              0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,
              0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,
              0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,
              0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
              0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
              0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,
              0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
              0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
              0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,2,
              0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,3,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,2,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,2,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,1,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,0,0,1,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,4,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,3,
              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3),nrow=50,ncol=26,byrow=T)
