
#------------------------------------------------------------------
# Simulated Salmon data
#------------------------------------------------------------------
y <- c(104,217,226,466,764,781,903,1356,1189,1522,1334,1378,1146,1155,873,1918,1916,1445,1078,1573)
T <- length(y)

# Winbugs model:
sink("salmon.bug")
cat("
    model {
    # Priors
    alpha    ~ dunif(1,2.5)
    beta     ~ dunif(0.00001,0.001)
    n.init   ~ dunif(50,500)
    sigma.sq <- 0.2
    tau.obs  <- 1/sigma.sq
    
    for (k in 1:K) {
    mu[k,1]    <- alpha*n.init*exp(-beta*n.init)
    state[k,1] ~ dpois(mu[k,1])
    temp[k,1]  <- log(state[k,1])-sigma.sq/2
    obs[k,1]   ~ dlnorm(temp[k,1],tau.obs)
    }
    
    for (k in 1:K) {
    for(i in 2:T) {
    mu[k,i]    <- alpha*state[k,i-1]*exp(-beta*state[k,i-1])
    state[k,i] ~  dpois(mu[k,i])
    temp[k,i]  <- log(state[k,i])-sigma.sq/2
    obs[k,i]   ~  dlnorm(temp[k,i],tau.obs)
    }
    }
    }
    ",fill=TRUE)
sink()

K <- 1
y2 <- matrix(rep(y,K),nrow=K,ncol=T,byrow=T)
data.or <-list(T=T,obs=y2,K=K)

inits <- function(){list(alpha=1.7,beta=0.0004,n.init=75,state=y2)}
parameters <- c("alpha","beta","n.init")
chain <- 1 
iter <- 50000
burn	<- 10000
thin <- 1
sal.out <- bugs(data=data.or,inits=inits,model.file = "salmon.bug",
                parameters=parameters, n.chains =chain, n.iter =iter,
                n.burnin=burn,n.thin=thin,debug=F, 
                bugs.directory =bugs.dir,clearWD=TRUE,codaPkg=TRUE)

